@@ -11,7 +11,7 @@ import os
 from ingradient_library.patch_transform import Batch_Affine_3D
 from ingradient_library.preprocessing import *
 from ingradient_library.transform import *
-from torchvision.transforms import RandomCrop
+from torchvision.transforms import CenterCrop
 import h5py
 
 
@@ -62,7 +62,7 @@ def set_direction(img, direction):
 
 
 class Dataset2D(Dataset):
-    def __init__(self, root_dir, axis, mode, hdf_img_keys = None, train = True, hdf_seg_key = None ,sampler = None, size = (520, 520),
+    def __init__(self, root_dir, axis, mode, hdf_img_keys = None, train = True, hdf_seg_key = None ,sampler = None, size = (512, 512), gru_size = (512,512,512),no_sampling = False,
                 oversampling = True, device = None, normalizer = None, transform = None, resample = None, correct_direction = True):
 
         self.file_name = []
@@ -91,7 +91,8 @@ class Dataset2D(Dataset):
         self.resample = resample
         self.correct_direction = correct_direction
         self.train = train
-        self.crop = RandomCrop(size = size, pad_if_needed = True)
+        self.crop = CenterCrop(size = size)
+        self.no_sampling = no_sampling
         
     def __len__(self):
         return len(self.file_name)
@@ -151,6 +152,8 @@ class Dataset2D(Dataset):
         if self.sampler != None:
             x, y = self.sampler(x, y)
         
+        
+
 
         if self.axis == None:
             return x, y
@@ -163,8 +166,9 @@ class Dataset2D(Dataset):
         
         else:
             x, y = random_axis_sample(x, y, self.axis, mode = 'random')
-
-        x, y = self.crop(image = x, target = y)
+    
+        x = self.crop(x)
+        y = self.crop(y)
         return x, y
 
         
@@ -198,26 +202,6 @@ class CustomDataset(Dataset):
 
     def __len__(self):
         return len(self.image_name)
-    
-    def set_direction(self, img, direction):
-        if isinstance(img, np.ndarray):
-            img = torch.tensor(img)
-        img = img.permute(2,1,0)
-        img = img.numpy()
-        direction = np.array(direction).reshape(3,3)
-        inverse = np.where(direction == -1)[0]
-        permute = np.where(direction != 0)[1]
-        img = img.transpose(permute)
-        if 0 in inverse:
-            img = img[::-1, :, :]
-
-        if 1 in inverse:
-            img = img[:, ::-1, :]
-
-        if 2 in inverse:
-            img = img[:, :, ::-1]
-
-        return torch.tensor(img.copy())
             
     def __getitem__(self, idx):
         info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
@@ -237,12 +221,12 @@ class CustomDataset(Dataset):
         if self.correct_direction:
             for i in range(len(x)):
                 if i == 0:
-                    result = self.set_direction(x[i], info_data['direction']).unsqueeze(0)
+                    result = set_direction(x[i], info_data['direction']).unsqueeze(0)
                 else:
-                    result = torch.vstack((result, self.set_direction(x[i], info_data['direction']).unsqueeze(0)))
+                    result = torch.vstack((result, set_direction(x[i], info_data['direction']).unsqueeze(0)))
             x = result 
             if self.mode == 'train' :
-                y = self.set_direction(y, info_data['direction'])
+                y = set_direction(y, info_data['direction'])
 
         if self.normalizer:
             x = self.normalizer(x)
@@ -639,18 +623,16 @@ class Interactive_DataLoader(DataLoader3D):
 
 class CustomDatasetV2(CustomDataset):
     def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None,
-                 patch_size = (128, 128, 128), transform = None, batch_size = 2):
+                 patch_size = (128, 128, 128), transform = None):
         
         self.patch_size = np.array(patch_size)
         self.transform = transform
-        self.batch_size = batch_size
+        self.batch_size = 1
         super().__init__(root_dir, device = device, normalizer = normalizer, mode = mode, dim = dim, correct_direction = correct_direction, resample = resample)
     
     def __len__(self):
         return super().__len__()
     
-    def set_direction(self, img, direction):
-        return super().set_direction(img, direction)
     
     def get_oversample_patch(self, images, seg):
         if not isinstance(images, np.ndarray):
@@ -735,10 +717,13 @@ class CustomDatasetV2(CustomDataset):
 
         if self.correct_direction:
             for i in range(len(x)):
-                x[i] = self.set_direction(x[i], info_data['direction'])
-            
+                if i == 0:
+                    result = set_direction(x[i], info_data['direction']).unsqueeze(0)
+                else:
+                    result = torch.vstack((result, set_direction(x[i], info_data['direction']).unsqueeze(0)))
+            x = result 
             if self.mode == 'train' :
-                y = self.set_direction(y, info_data['direction'])
+                y = set_direction(y, info_data['direction'])
 
         if self.normalizer:
             x = self.normalizer(x)
@@ -746,7 +731,8 @@ class CustomDatasetV2(CustomDataset):
         
         if self.resample != None:
             if self.mode == 'train' :
-                y = torch.tensor(y.copy())
+                if isinstance(y, np.ndarray):
+                    y = torch.tensor(y.copy())
                 result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
                 x = result[:-1]
                 y = result[-1].squeeze(0)
@@ -790,7 +776,7 @@ class CustomDatasetV2(CustomDataset):
             result_images, result_seg = self.transform(result_images.to(self.device), result_seg.to(self.device), info_data)
         
 
-        return result_images, result_seg.long()
+        return result_images.squeeze(0), result_seg.squeeze(0).long(), info_data
 
     
 
