@@ -48,6 +48,20 @@ class ConvDropoutNormNonlin(nn.Module):
         return self.lrelu(self.instnorm(x))
 
 
+class DeepLabV3_CT(nn.Module):
+    def __init__(self, in_channels, out_channels):
+        super().__init__()
+        self.deeplab = torch.hub.load('pytorch/vision:v0.10.0', 'deeplabv3_resnet50', pretrained=True)
+        self.deeplab.backbone.conv1 = nn.Conv2d(in_channels, 64, kernel_size=(7, 7), stride=(2, 2), padding=(3, 3), bias=False)
+        self.deeplab.aux_classifier[-1] = nn.Conv2d(512, out_channels, kernel_size=(1,1), stride=(1,1))
+        
+
+    def forward(self, input, hidden):
+        
+        input = self.deeplab.backbone(input)
+        input = self.deeplab.classifier(input)
+
+
 
 class DeepSuperVisionUNet2dot5(nn.Module):
     def __init__(self, n_modalities = 1, depth = 256, fianl_output_channels = 2, patch_size = (256,512,512)):
@@ -146,16 +160,16 @@ class Deep_Supervision_UNet3d_Narrow(nn.Module):
 
 
         self.final_layer1 = nn.Sequential(
-            nn.Conv3d(32, final_output_channels, 1, 1, 0) #DeepLab Architecture로 수정
+            nn.Conv3d(32, final_output_channels, 3, 1, 1)
             )
         self.final_layer2 = nn.Sequential(
-            nn.Conv3d(64, final_output_channels, 1, 1, 0)
+            nn.Conv3d(64, final_output_channels, 3, 1, 1)
             )
         self.final_layer3 = nn.Sequential(
-            nn.Conv3d(128, final_output_channels, 1, 1, 0)
+            nn.Conv3d(128, final_output_channels, 3, 1, 1)
             )
         self.final_layer4 = nn.Sequential(
-            nn.Conv3d(256, final_output_channels, 1, 1, 0)
+            nn.Conv3d(256, final_output_channels, 3, 1, 1)
             )
 
         self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
@@ -181,10 +195,10 @@ class Deep_Supervision_UNet3d_Narrow(nn.Module):
             x3_ = F.softmax(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
             x4_ = F.softmax(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
         else:
-            x1_ = F.sigmoid(self.final_layer1(x1_).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
-            x2_ = F.sigmoid(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
-            x3_ = F.sigmoid(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
-            x4_ = F.sigmoid(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x1_ = torch.sigmoid(self.final_layer1(x1_).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x2_ = torch.sigmoid(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x3_ = torch.sigmoid(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x4_ = torch.sigmoid(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
         
         result = torch.stack([x1_, x2_, x3_, x4_]).permute(1,0,2,3,4,5)
         
@@ -257,16 +271,16 @@ class Deep_Supervision_UNet3d(nn.Module):
         )
 
         self.final_layer1 = nn.Sequential(
-            nn.Conv3d(32, final_output_channels, 1, 1, 0) #DeepLab Architecture로 수정
+            nn.Conv3d(32, final_output_channels, kernel_size = 1, stride = 1, padding = 0)
             )
         self.final_layer2 = nn.Sequential(
-            nn.Conv3d(64, final_output_channels, 1, 1, 0)
+            nn.Conv3d(64, final_output_channels, kernel_size = 1, stride = 1, padding = 0)
             )
         self.final_layer3 = nn.Sequential(
-            nn.Conv3d(128, final_output_channels, 1, 1, 0)
+            nn.Conv3d(128, final_output_channels, kernel_size = 1, stride = 1, padding = 0)
             )
         self.final_layer4 = nn.Sequential(
-            nn.Conv3d(256, final_output_channels, 1, 1, 0)
+            nn.Conv3d(256, final_output_channels, kernel_size = 1, stride = 1, padding = 0)
             )
 
         self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
@@ -292,11 +306,13 @@ class Deep_Supervision_UNet3d(nn.Module):
             x2_ = F.softmax(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
             x3_ = F.softmax(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
             x4_ = F.softmax(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1), dim = 1).view(bs, self.final_output_channels, nx, ny, nz)
+        
+
         else:
-            x1_ = F.sigmoid(self.final_layer1(x1_).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
-            x2_ = F.sigmoid(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
-            x3_ = F.sigmoid(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
-            x4_ = F.sigmoid(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x1_ = torch.sigmoid(self.final_layer1(x1_).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x2_ = torch.sigmoid(self.upsample2(self.final_layer2(x2_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x3_ = torch.sigmoid(self.upsample3(self.final_layer3(x3_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
+            x4_ = torch.sigmoid(self.upsample4(self.final_layer4(x4_)).view(bs, self.final_output_channels, -1)).view(bs, self.final_output_channels, nx, ny, nz)
         
         result = torch.stack([x1_, x2_, x3_, x4_]).permute(1,0,2,3,4,5)
         
@@ -306,94 +322,7 @@ class Deep_Supervision_UNet3d(nn.Module):
         min_patch_axis = np.min(self.patch_size)
         return np.clip(np.log2(min_patch_axis), 0, 6).astype(int)
     
-class Deep_Supervision_UNet3d_Lung(nn.Module):
-    def __init__(self, n_modalities = 1, final_output_channels = 2, patch_size = (128, 128, 128), is_binary = False):
-        super().__init__()
-        self.patch_size = np.array(patch_size)
-        self.networks_depth = self.get_networks_depth()
-        self.encoder_blocks = []
-        self.decoder_blocks = []
-        self.upsample_operations = []
-        self.final_output_channels = final_output_channels
-        self.is_binary = is_binary
-        
-        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
-        self.encoder1 = nn.Sequential(
-            ConvDropoutNormNonlin(n_modalities, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(32, 32, conv_kwargs = conv_kwargs1, conv_op = nn.Conv3d)
-        )
-        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 2}
-        conv_kwargs2 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
-        self.encoder2 = nn.Sequential(
-            ConvDropoutNormNonlin(32, 64, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(64, 64, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.encoder3 = nn.Sequential(
-            ConvDropoutNormNonlin(64, 128, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(128, 128, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.encoder4 = nn.Sequential(
-            ConvDropoutNormNonlin(128, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(256, 256, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.encoder5 = nn.Sequential(
-            ConvDropoutNormNonlin(256, 320, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(320, 320, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
 
-        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1,  'stride' : 1}
-        conv_kwargs2 = {'kernel_size' : 3, 'stride' : 1, 'padding':1}
-        transp_kwargs = {'kernel_size' : 2, 'stride' : 1, 'stride' : 2}
-
-
-        self.decoder1_trans = ConvDropoutNormNonlin(320, 256, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
-        self.decoder1_conv = nn.Sequential(
-            ConvDropoutNormNonlin(256*2, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(256, 256, conv_kwargs = conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.decoder2_trans = ConvDropoutNormNonlin(256, 128, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
-        self.decoder2_conv = nn.Sequential(
-            ConvDropoutNormNonlin(128*2, 128 , conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(128, 128, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.decoder3_trans = ConvDropoutNormNonlin(128, 64, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
-        self.decoder3_conv = nn.Sequential(
-            ConvDropoutNormNonlin(64*2, 64, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(64, 64, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.decoder4_trans = ConvDropoutNormNonlin(64, 32, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
-        self.decoder4_conv = nn.Sequential(
-            ConvDropoutNormNonlin(32*2, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(32, 32, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
-        )
-
-        self.final_block = nn.Sequential(
-            ConvDropoutNormNonlin(32, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
-            ConvDropoutNormNonlin(32, 32, conv_kwargs = conv_kwargs2, conv_op = nn.Conv3d)
-        )
-        self.final_layer = nn.Sequential(
-            nn.Conv3d(32, final_output_channels, 1, 1, 0)
-        )
-        
-    
-    def forward(self, x):
-        x1 = self.encoder1(x)
-        x2 = self.encoder2(x1)
-        x3 = self.encoder3(x2)
-        x4 = self.encoder4(x3)
-        x5 = self.encoder5(x4)
-
-        x4_ = self.decoder1_conv(torch.cat((x4, self.decoder1_trans(x5)), dim = 1))
-        x3_ = self.decoder2_conv(torch.cat((x3, self.decoder2_trans(x4_)), dim = 1))
-        x2_ = self.decoder3_conv(torch.cat((x2, self.decoder3_trans(x3_)), dim = 1))
-        x1_ = self.decoder4_conv(torch.cat((x1, self.decoder4_trans(x2_)), dim = 1))
-        x1_ = self.final_block(x1_)
-        
-        return self.final_layer(x1_)
-                
-    def get_networks_depth(self):
-        min_patch_axis = np.min(self.patch_size)
-        return np.clip(np.log2(min_patch_axis), 0, 6).astype(int)
 
 class Interactive_Model(nn.Module):
     def __init__(self, n_modalities = 1, point_channel = 2, patch_size = (320,256,256), final_output_channels = 1):
@@ -440,6 +369,75 @@ class Interactive_Model(nn.Module):
 
 
 
+class Daegu_Model10(nn.Module):
+    def __init__(self, patch_size, resnet_output = 4):
+        super().__init__()
+        resnet_output = resnet_output
+        out_tconv1 = resnet_output//2
+        out_tconv2 = out_tconv1//2
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':resnet_output}
+        self.model = resnet10(**kwargs)
+
+        self.upsample1 = nn.Sequential(nn.ConvTranspose3d(in_channels = resnet_output, out_channels= out_tconv1, kernel_size =3 , stride = 2, padding = 1, output_padding =1),
+                                       nn.InstanceNorm3d(out_tconv1),
+                                       nn.LeakyReLU())
+        
+        self.upsample2 = nn.Sequential(nn.ConvTranspose3d(in_channels = out_tconv1, out_channels= out_tconv2, kernel_size =3 , stride = 2, padding = 1, output_padding =1),
+                                       nn.InstanceNorm3d(out_tconv2),
+                                       nn.LeakyReLU())
+        
+        self.final_layer = nn.Sequential(nn.Conv3d(in_channels = out_tconv2, out_channels = 2, kernel_size = 3, stride = 1, padding = 1),
+                                   nn.InstanceNorm3d(2),
+                                   nn.LeakyReLU())
+
+    def forward(self, X):
+        X = self.model(X)
+        X = self.upsample1(X)
+        X = self.upsample2(X)
+        X = self.final_layer(X)
+        return torch.softmax(X, dim = -1)
+
+class Daegu_Model34(nn.Module):
+    def __init__(self, patch_size, input_channel = 1, output_channel= 2):
+        super().__init__()
+        out_channel1 = 1
+        out_channel2 = out_channel1 * 4
+
+        self.a_conv1 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 3, stride = 1, padding = 6, dilation = 6),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+        self.a_conv2 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 3, stride = 1, padding = 12, dilation = 12),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+        self.a_conv3 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 3, stride = 1, padding = 18, dilation = 18),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+        self.a_conv4 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+
+        self.a_conv2_1 = nn.Sequential(nn.Conv3d(in_channels = out_channel2, out_channels = out_channel2, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                                   nn.InstanceNorm3d(out_channel2),
+                                   nn.LeakyReLU())
+
+        self.last_layer = nn.Sequential(nn.Conv3d(in_channels = out_channel2, out_channels = output_channel, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                                   nn.InstanceNorm3d(output_channel),
+                                   nn.LeakyReLU())
+
+
+    def forward(self, X):
+        X1 = self.a_conv1(X)
+        X2 = self.a_conv2(X)
+        X3 = self.a_conv3(X)
+        X4 = self.a_conv4(X)
+
+        X = torch.cat((X1,X2,X3,X4), dim = 1)
+        X = self.a_conv2_1(X)
+        X = self.last_layer(X)
+        return torch.softmax(X, dim = -1)
+
+
+
 
 def conv3x3x3(in_planes, out_planes, stride=1, dilation=1):
     # 3x3x3 convolution with padding
@@ -473,10 +471,10 @@ class BasicBlock(nn.Module):
     def __init__(self, inplanes, planes, stride=1, dilation=1, downsample=None):
         super(BasicBlock, self).__init__()
         self.conv1 = conv3x3x3(inplanes, planes, stride=stride, dilation=dilation)
-        self.bn1 = nn.InstanceNorm3d(planes)
+        self.bn1 = nn.BatchNorm3d(planes)
         self.relu = nn.LeakyReLU(inplace = True)
         self.conv2 = conv3x3x3(planes, planes, dilation=dilation)
-        self.bn2 = nn.InstanceNorm3d(planes)
+        self.bn2 = nn.BatchNorm3d(planes)
         self.downsample = downsample
         self.stride = stride
         self.dilation = dilation
@@ -505,12 +503,12 @@ class Bottleneck(nn.Module):
     def __init__(self, inplanes, planes, stride=1, dilation=1, downsample=None):
         super(Bottleneck, self).__init__()
         self.conv1 = nn.Conv3d(inplanes, planes, kernel_size=1, bias=False)
-        self.bn1 = nn.InstanceNorm3d(planes)
+        self.bn1 = nn.BatchNorm3d(planes)
         self.conv2 = nn.Conv3d(
             planes, planes, kernel_size=3, stride=stride, dilation=dilation, padding=dilation, bias=False)
-        self.bn2 = nn.InstanceNorm3d(planes)
+        self.bn2 = nn.BatchNorm3d(planes)
         self.conv3 = nn.Conv3d(planes, planes * 4, kernel_size=1, bias=False)
-        self.bn3 = nn.InstanceNorm3d(planes * 4)
+        self.bn3 = nn.BatchNorm3d(planes * 4)
         self.relu = nn.LeakyReLU(inplace=True)
         self.downsample = downsample
         self.stride = stride
@@ -561,7 +559,7 @@ class ResNet(nn.Module):
             padding=(3, 3, 3),
             bias=False)
             
-        self.bn1 = nn.InstanceNorm3d(64)
+        self.bn1 = nn.BatchNorm3d(64)
         self.relu = nn.LeakyReLU(inplace=True)
         self.maxpool = nn.MaxPool3d(kernel_size=(3, 3, 3), stride=2, padding=1)
         self.layer1 = self._make_layer(block, 64, layers[0], shortcut_type)
@@ -579,7 +577,7 @@ class ResNet(nn.Module):
                                         2,
                                         stride=2
                                         ),
-                                        nn.InstanceNorm3d(32),
+                                        nn.BatchNorm3d(32),
                                         nn.LeakyReLU(inplace=True),
                                         nn.Conv3d(
                                         32,
@@ -588,7 +586,7 @@ class ResNet(nn.Module):
                                         stride=(1, 1, 1),
                                         padding=(1, 1, 1),
                                         bias=False), 
-                                        nn.InstanceNorm3d(32),
+                                        nn.BatchNorm3d(32),
                                         nn.LeakyReLU(inplace=True),
                                         nn.Conv3d(
                                         32,
@@ -618,7 +616,7 @@ class ResNet(nn.Module):
                         planes * block.expansion,
                         kernel_size=1,
                         stride=stride,
-                        bias=False), nn.InstanceNorm3d(planes * block.expansion))
+                        bias=False), nn.BatchNorm3d(planes * block.expansion))
 
         layers = []
         layers.append(block(self.inplanes, planes, stride=stride, dilation=dilation, downsample=downsample))
